#include <lemon/list_graph.h>
#include <lemon/lgf_reader.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <iomanip>
#include <string>
#include <math.h>
#include <vector>
#include <lemon/lp.h>


#define EPS 0.0001

using namespace lemon;
using namespace std;

void readGraph(	string filename, ListGraph& G,
				ListGraph::EdgeMap<int>& label){

	// Le arquivo.
	ifstream file(filename.c_str());
	stringstream input;
	if(file){
		input << file.rdbuf();
		file.close();
	} else {
		cout << "File not found: " << filename << endl;
		exit(1);
	}
	
	// Parser.
	GraphReader<ListGraph>(G, input).edgeMap("label", label).run();
}

void solveEdgeColoring( ListGraph& G,
			 ListGraph::EdgeMap<int>& label, 
			 ListGraph::EdgeMap<int>& coloring){

	Mip lp;
	int numEdges = countEdges(G);
	
	// Grau Maximo.
	int max_degree = 0;
	for(ListGraph::NodeIt v(G); v != INVALID; ++v){
		int degree = countOutArcs(G, v);
		if(degree > max_degree){max_degree = degree;}
	}
	int K = max_degree + 1;
	
    // Variavel x[i][j] = 1 se aresta i tem cor j.
    Mip::Col **x = 0;

    // Criando as variaveis.
    x = new Mip::Col* [numEdges];
    for(int i = 0; i < numEdges; ++i){
    	x[i] = new Lp::Col [K];
    	for (int j = 0; j < K; ++j){
    		x[i][j] = lp.addCol();
    		lp.colType(x[i][j], Mip::INTEGER);
    		lp.colLowerBound(x[i][j], 0 );
    		lp.colUpperBound(x[i][j], 1 );
    	}
    }

    // Restricao 1: todo vertice tem exatamente uma cor.
    for(int i = 0; i < numEdges; ++i){
    	Mip::Expr e = 0;
    	for (int j = 0; j < K; ++j){
    		e += x[i][j];
    	}
    	lp.addRow(e == 1);
    }

    //Para cada no.
    for (ListGraph::NodeIt n(G); n!=INVALID; ++n){
    	//Para cada cor.
    	for (int j = 0; j < K; ++j){
    		Mip::Expr e = 0;
    		// Restricao 2: a soma das arestas que usam uma cor eh menor ou igual a 1.
			for (ListGraph::IncEdgeIt edge(G, n); edge != INVALID; ++edge) {
				e += x[label[edge]][j];
			}
        	lp.addRow(e <= 1);
        }
    }

	// Resolve pli.
    lp.solve();
	
	// Transcreve solucao.
	for(ListGraph::EdgeIt e(G); e != INVALID; ++e){
		int i = label[e];
		for (int j = 0; j < K; ++j){
        	if (lp.sol(x[i][j]) > 1.0-EPS){coloring[e] = j;}
        }
	}
}

int main(int argc, char *argv[]){

	if(argc != 2){
		cout << "uso: " << argv[0] << " <filename>" << endl;
		return 1;
	}

	// Le o grafo.
	string filename = argv[1];	
	
	ListGraph G;
	ListGraph::EdgeMap<int> label(G);	
	
	readGraph(filename, G, label);
	
	// Encontra uma (max_degree+1)-coloracao das arestas.
	ListGraph::EdgeMap<int> coloring(G);
	solveEdgeColoring(G, label, coloring);
	
	// Imprime (max_degree+1)-coloracao.
	for(ListGraph::EdgeIt e(G); e != INVALID; ++e){
		cout << label[e] << "\t" << coloring[e] << "\n";
	}

	return 0;
}
